import 'dart:async';

import 'package:aj_captcha_flutter/aj_captcha_flutter.dart';
import 'package:aj_captcha_flutter/captcha/aj_captcha_init.dart';
import 'package:aj_captcha_flutter/captcha/view/block_puzzle_captcha.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

void main() {
  AJCaptchaInit.init("http://127.0.0.1:80/");
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  final _ajCaptchaFlutterPlugin = AjCaptchaFlutter();

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    // We also handle the message potentially returning null.
    try {
      platformVersion = await _ajCaptchaFlutterPlugin.getPlatformVersion() ??
          'Unknown platform version';
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  void loadingBlockPuzzle(BuildContext buildContext,
      {barrierDismissible = true}) {
    showDialog<void>(
      context: buildContext,
      barrierDismissible: barrierDismissible,
      builder: (BuildContext buildContext) {
        return BlockPuzzleCaptchaPage(
          onSuccess: (v) {
            //成功回调（回调为加密后内容，依据项目而定）
            debugPrint(v.toString());
          },
          onFail: () {},
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Builder(
          builder:(context){
            return  Center(
              child: Column(
                children: [
                  Text('Running on: $_platformVersion\n'),
                  TextButton(
                      onPressed: () {
                        loadingBlockPuzzle(context);
                      },
                      child: Text("点击开始验证"))
                ],
              ),
            );
          },
        ),
      ),
    );
  }
}
